# SVGfix
#
# Copyright, Jacques Distler & Sam Ruby, 2007-2012.
# Licensed under the Perl Artistic License.
#
# Version: 0.4
#
# Adds a global filter to "fix" Namespace Well-fomed XHTML+MathML+SVG to conform
# to the W3C profile (which demands that the SVG be prefixed).
#
# Usage:
#
#        <MTEntryBody svg_fix="1">
#

package MT::Plugin::SVGfix;

use vars qw($VERSION);
use XML::Parser;
use MathML::Entities;
use strict;
$VERSION = '0.4';

eval{ require MT::Plugin };
unless ($@) {
    my $plugin = {
        name => "SVGfix $VERSION",
        description => 'Coerce Namespace Well-Formed XHTML+MathML+SVG into a form the W3C Validator will smile upon.',
        doc_link => 'http://golem.ph.utexas.edu/~distler/blog/SVGfix.html'
    }; 
    MT->add_plugin(new MT::Plugin($plugin));
}

MT::Template::Context->add_global_filter('svg_fix', \&svgprefix);

use constant SVGNS => 'http://www.w3.org/2000/svg';
use constant XLINKNS => 'http://www.w3.org/1999/xlink';
use constant MATHMLNS => 'http://www.w3.org/1998/Math/MathML';
use constant EMPTYELEMENTS => qw( area base br col command embed hr img input keygen link meta param source track wbr mspace maligngroup malignmark msline none );

sub xmlEscape {
  $_ = shift;
  s/&/&amp;/g;
  s/</&lt;/g;
  s/>/&gt;/g;
  return $_;
}

sub xmlSerialize {
  my ($expat,$element,@attrs) = @_;
  $element = "svg:$element" if $expat->namespace($element) eq SVGNS;
  while (@attrs) {
    my ($name, $value) = (shift(@attrs), xmlEscape(shift(@attrs)));
    $name = "xlink:$name" if $expat->namespace($name) eq XLINKNS;
    $name = "xml:lang" if $name eq "lang";
    $value =~ s/'/&#x27;/g;
    $element .= " $name='$value'";
  }
  return $element;
}

sub startHandler {
  my ($expat,$element,@attrs) = @_;
  $element .= " xmlns=\"${\MATHMLNS}\"" 
    if $element eq 'math' && $expat->namespace($element) eq MATHMLNS;
  return '<svg:switch><' . xmlSerialize($expat,$element,@attrs) . '>'
    if $element eq 'foreignObject' && $expat->current_element ne 'switch';
  foreach my $empty (EMPTYELEMENTS) {
    return '<' . xmlSerialize($expat,$element,@attrs) . ' />'
      if $element eq $empty;
  }
  return '<' . xmlSerialize($expat,$element,@attrs) . '>' ;
}

sub endHandler {
  my ($expat,$element,@attrs) = @_;
  return '</' . xmlSerialize($expat,$element,@attrs) . '></svg:switch>'
    if $element eq 'foreignObject' && $expat->current_element ne 'switch';
  foreach my $empty (EMPTYELEMENTS) {
    return '' if $element eq $empty;
  }
  return '</' .  xmlSerialize($expat,$element,@attrs) . '>';
}

sub svgprefix {
  my $input = shift;
  $input = MathML::Entities::name2numbered($input);
  my $publish_charset = MT::ConfigMgr->instance->PublishCharset;
  my $output = '';
  my $parser = new XML::Parser(Namespaces => 1);
  $parser->setHandlers(
    Char  => sub {shift; $output .= xmlEscape(shift)},
    Start => sub {$output .= startHandler(@_) },
    End   => sub {$output .= endHandler(@_) },
  );
  eval {$parser->parse("<div xmlns=\"http://www.w3.org/1999/xhtml\">". $input ."</div>",
            ProtocolEncoding => uc($publish_charset))} or return $input;
  if ($output =~ / xlink:/) {
    $output =~ s/<math/$& xmlns:xlink="${\XLINKNS}"/g;
    $output =~ s/<svg:svg/$& xmlns:xlink="${\XLINKNS}"/g;
  }
  $output =~ s/<svg:svg/$& xmlns:svg="${\SVGNS}"/g;
  $output =~ s/^<div xmlns="http:\/\/www.w3.org\/1999\/xhtml">(.*)<\/div>$/$1/s;
# FIXME: We should be encoding to $publish_charset, not 'ascii'. But that fails when
# $output contains characters in the repertoir of $publish_charset. The utf8 bytes
# are passed through, instead of re-encoded. Thus U+00D7, whose utf-8 encoding is 0xC397
# becomes two characters U+00C3, followed by U+0097.
  $output = Encode::encode('ascii', $output, Encode::FB_XMLCREF)
      unless $publish_charset eq 'utf-8';
  return $output;
}

1;
